@@ -23,7 +23,7 @@
 #include <math.h>
 
 #include <osmscout/AdminRegion.h>
-#include <osmscout/Database.h>
+#include "osmscout/database/Database.h"
 #include <osmscout/Node.h>
 #include <osmscoutmap/MapPainter.h>
 
