@@ -23,6 +23,9 @@
 
 #include <osmscout/system/Math.h>
 
+#include <QtMath>
+#include <QDebug>
+
 namespace osmscout {
 
 // uncomment or define by compiler parameter to render various debug marks
@@ -82,12 +85,17 @@ PlaneMapRenderer::~PlaneMapRenderer()
     delete loadJob;
 }
 
+void PlaneMapRenderer::onDatabaseLoaded([[maybe_unused]] osmscout::GeoBox boundingBox)
+{
+  InvalidateVisualCache();
+}
+
 void PlaneMapRenderer::InvalidateVisualCache()
 {
   {
     QMutexLocker finishedLocker(&finishedMutex);
     osmscout::log.Debug() << "Invalidate finished image";
-    epoch++;;
+    epoch++;
   }
   emit Redraw();
 }
@@ -164,17 +172,17 @@ bool PlaneMapRenderer::RenderMap(QPainter& painter,
                          finalImgProjection.GetWidth(),
                          finalImgProjection.GetHeight());
 
-  double targetCenterX;
-  double targetCenterY;
+  osmscout::Vertex2D targetCenter;
 
   osmscout::GeoCoord srcImageCenterCoord;
   finalImgProjection.PixelToGeo(finalImgProjection.GetWidth()/2,
                                 finalImgProjection.GetHeight()/2,
                                 srcImageCenterCoord);
 
-  requestProjection.GeoToPixel(srcImageCenterCoord,targetCenterX,targetCenterY);
-  double targetTopLeftX=targetCenterX - finalImgProjection.GetWidth()*scale*0.5;
-  double targetTopLeftY=targetCenterY - finalImgProjection.GetHeight()*scale*0.5;
+  requestProjection.GeoToPixel(srcImageCenterCoord,
+                               targetCenter);
+  double targetTopLeftX=targetCenter.GetX() - finalImgProjection.GetWidth()*scale*0.5;
+  double targetTopLeftY=targetCenter.GetY() - finalImgProjection.GetHeight()*scale*0.5;
 
   QRectF targetRectangle(targetTopLeftX,
                          targetTopLeftY,
@@ -281,14 +289,14 @@ double PlaneMapRenderer::computeScale(const osmscout::MercatorProjection &previo
   currentProjection.PixelToGeo(currentProjection.GetWidth(),currentProjection.GetHeight(),
                                bottomRight);
 
-  double x1;
-  double y1;
-  previousProjection.GeoToPixel(topLeft,x1,y1);
-  double x2;
-  double y2;
-  previousProjection.GeoToPixel(bottomRight,x2,y2);
+  osmscout::Vertex2D pos1;
+  osmscout::Vertex2D pos2;
+  previousProjection.GeoToPixel(topLeft,
+                                pos1);
+  previousProjection.GeoToPixel(bottomRight,
+                                pos2);
 
-  double previousDiagonal=sqrt(pow(x2-x1,2) + pow(y2-y1,2));
+  double previousDiagonal=sqrt(pow(pos2.GetX()-pos1.GetX(),2) + pow(pos2.GetY()-pos1.GetY(),2));
   return currentDiagonal / previousDiagonal;
 }
 
@@ -330,8 +338,8 @@ void PlaneMapRenderer::DrawMap()
     drawParameter.SetPatternMode(osmscout::MapParameter::PatternMode::Scalable);
     drawParameter.SetIconPaths(paths);
     drawParameter.SetPatternPaths(paths);
-    drawParameter.SetDebugData(false);
-    drawParameter.SetDebugPerformance(true);
+    drawParameter.SetDebugData(osmscout::log.IsDebug());
+    drawParameter.SetDebugPerformance(osmscout::log.IsWarn());
     // We want to get notified, if we have more than 1000 objects from a certain type (=> move type rendering to a higher zoom level?)
     drawParameter.SetWarningObjectCountLimit(1000);
     // We want to get notified, if we have more than 20000 coords from a certain type (=> move type rendering to a higher zoom level?)
@@ -379,8 +387,7 @@ void PlaneMapRenderer::DrawMap()
 
     // overlay objects
     std::vector<OverlayObjectRef> overlayObjects;
-    osmscout::GeoBox renderBox;
-    projection.GetDimensions(renderBox);
+    osmscout::GeoBox renderBox(projection.GetDimensions());
     getOverlayObjects(overlayObjects, renderBox);
 
     bool success;
@@ -392,7 +399,7 @@ void PlaneMapRenderer::DrawMap()
                       overlayObjects,
                       dbThread->GetEmptyStyleConfig(),
                       /*drawCanvasBackground*/ true);
-      dbThread->RunJob(&job);
+      dbThread->RunJob(std::bind(&DBRenderJob::Run, &job, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
       success=job.IsSuccess();
     }
 
@@ -406,6 +413,7 @@ void PlaneMapRenderer::DrawMap()
 
     if (loadJob->IsFinished()){
       // this slot is may be called from DBLoadJob, we can't delete it now
+      loadJob->Close();
       loadJob->deleteLater();
       loadJob=nullptr;
     }
@@ -467,7 +475,8 @@ void PlaneMapRenderer::TriggerMapRendering(const MapViewStruct& request, size_t
   {
     QMutexLocker locker(&lock);
     if (loadJob!=nullptr){
-      // TODO: check if job contains same tiles...
+      // TODO: check if job contains the same tiles...
+      loadJob->Close();
       loadJob->deleteLater();
       loadJob=nullptr;
     }
@@ -505,7 +514,7 @@ void PlaneMapRenderer::TriggerMapRendering(const MapViewStruct& request, size_t
     connect(loadJob, &DBLoadJob::finished,
             this, &PlaneMapRenderer::onLoadJobFinished);
 
-    dbThread->RunJob(loadJob);
+    dbThread->RunJob(std::bind(&DBLoadJob::Run, loadJob, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
   }
   emit TriggerInitialRendering();
 }
