@@ -28,7 +28,7 @@
 #include <QVariant>
 
 #include <osmscout/routing/RouteDescription.h>
-#include <osmscout/util/Logger.h>
+#include <osmscout/log/Logger.h>
 #include <osmscout/util/String.h>
 
 namespace osmscout {
