@@ -23,15 +23,14 @@
 
 #include <osmscout/Node.h>
 
-#include <osmscout/AreaNodeIndex.h>
-#include <osmscout/NodeDataFile.h>
+#include <osmscout/db/AreaNodeIndex.h>
+#include <osmscout/db/NodeDataFile.h>
 
 #include <osmscout/system/Assert.h>
-#include <osmscout/system/Math.h>
 
-#include <osmscout/util/File.h>
-#include <osmscout/util/FileWriter.h>
-#include <osmscout/util/FileScanner.h>
+#include <osmscout/io/File.h>
+#include <osmscout/io/FileWriter.h>
+#include <osmscout/io/FileScanner.h>
 #include <osmscout/util/Geometry.h>
 #include <osmscout/util/Number.h>
 
