@@ -28,21 +28,28 @@
 
 #include <osmscout/Pixel.h>
 
-#include <osmscout/TypeFeatures.h>
 #include <osmscout/FeatureReader.h>
 
-#include <osmscout/LocationIndex.h>
+#include <osmscout/db/LocationIndex.h>
 
-#include <osmscout/AreaDataFile.h>
-#include <osmscout/NodeDataFile.h>
-#include <osmscout/WayDataFile.h>
+#include <osmscout/db/AreaDataFile.h>
+#include <osmscout/db/NodeDataFile.h>
+#include <osmscout/db/WayDataFile.h>
+
+#include <osmscout/feature/AdminLevelFeature.h>
+#include <osmscout/feature/IsInFeature.h>
+#include <osmscout/feature/NameFeature.h>
+#include <osmscout/feature/NameAltFeature.h>
+#include <osmscout/feature/PostalCodeFeature.h>
+#include <osmscout/feature/RefFeature.h>
 
 #include <osmscout/system/Assert.h>
 #include <osmscout/system/Math.h>
 
-#include <osmscout/util/File.h>
-#include <osmscout/util/FileScanner.h>
-#include <osmscout/util/FileWriter.h>
+#include <osmscout/io/File.h>
+#include <osmscout/io/FileScanner.h>
+#include <osmscout/io/FileWriter.h>
+
 #include <osmscout/util/GeoBox.h>
 #include <osmscout/util/Geometry.h>
 
