@@ -21,11 +21,12 @@
 
 #include <numeric>
 
-#include <osmscout/util/File.h>
-#include <osmscout/util/FileScanner.h>
-#include <osmscout/util/FileWriter.h>
-#include <osmscout/util/ProcessingQueue.h>
-#include <osmscout/util/Worker.h>
+#include <osmscout/io/File.h>
+#include <osmscout/io/FileScanner.h>
+#include <osmscout/io/FileWriter.h>
+
+#include <osmscout/async/ProcessingQueue.h>
+#include <osmscout/async/Worker.h>
 
 #include <osmscoutimport/MergeAreaData.h>
 
