@@ -21,21 +21,22 @@
 
 #include <osmscout/Pixel.h>
 
-#include <osmscout/TypeFeatures.h>
+#include <osmscout/feature/RefFeature.h>
 
 #include <osmscout/Way.h>
 
-#include <osmscout/WayDataFile.h>
-#include <osmscout/OptimizeWaysLowZoom.h>
+#include <osmscout/db/WayDataFile.h>
+#include <osmscout/db/OptimizeWaysLowZoom.h>
 
 #include <osmscout/system/Assert.h>
-#include <osmscout/system/Math.h>
 
-#include <osmscout/util/File.h>
+#include <osmscout/projection/MercatorProjection.h>
+
+#include <osmscout/io/File.h>
+
 #include <osmscout/util/Geometry.h>
 #include <osmscout/util/GeoBox.h>
 #include <osmscout/util/Number.h>
-#include <osmscout/util/Projection.h>
 #include <osmscout/util/TileId.h>
 #include <osmscout/util/Transformation.h>
 
