@@ -268,7 +268,7 @@ namespace osmscout {
 
   void MapPainterAgg::DrawLabel(const Projection& /*projection*/,
                                 const MapParameter& /*parameter*/,
-                                const DoubleRectangle& labelRectangle,
+                                const ScreenVectorRectangle& labelRectangle,
                                 const LabelData& label,
                                 const NativeLabel& layout)
   {
@@ -339,10 +339,10 @@ namespace osmscout {
     }
   }
 
-  osmscout::DoubleRectangle MapPainterAgg::GlyphBoundingBox(const NativeGlyph &glyph) const
+  osmscout::ScreenVectorRectangle MapPainterAgg::GlyphBoundingBox(const NativeGlyph &glyph) const
   {
-    return DoubleRectangle(glyph.aggGlyph->bounds.x1, glyph.aggGlyph->bounds.y1,
-                           glyph.aggGlyph->bounds.x2, glyph.aggGlyph->bounds.y2);
+    return ScreenVectorRectangle(glyph.aggGlyph->bounds.x1, glyph.aggGlyph->bounds.y1,
+                                 glyph.aggGlyph->bounds.x2, glyph.aggGlyph->bounds.y2);
   }
 
   template<>
@@ -353,7 +353,7 @@ namespace osmscout {
     for (const auto &nativeGlyph : label.glyphs) {
       MapPainterAgg::AggGlyph glyph{};
       glyph.glyph=nativeGlyph;
-      glyph.position.Set(nativeGlyph.x, nativeGlyph.y);
+      glyph.position=Vertex2D(nativeGlyph.x, nativeGlyph.y);
       result.emplace_back(glyph);
     }
     return result;
@@ -407,11 +407,17 @@ namespace osmscout {
 
   void MapPainterAgg::RegisterRegularLabel(const Projection &projection,
                                            const MapParameter &parameter,
+                                           const ObjectFileRef& ref,
                                            const std::vector<LabelData> &labels,
                                            const Vertex2D &position,
                                            double objectWidth)
   {
-    labelLayouter.RegisterLabel(projection, parameter, position, labels, objectWidth);
+    labelLayouter.RegisterLabel(projection,
+                                parameter,
+                                ref,
+                                position,
+                                labels,
+                                objectWidth);
   }
 
   /**
@@ -419,10 +425,15 @@ namespace osmscout {
    */
   void MapPainterAgg::RegisterContourLabel(const Projection &projection,
                                            const MapParameter &parameter,
+                                           const ObjectFileRef& ref,
                                            const PathLabelData &label,
                                            const LabelPath &labelPath)
   {
-    labelLayouter.RegisterContourLabel(projection, parameter, label, labelPath);
+    labelLayouter.RegisterContourLabel(projection,
+                                       parameter,
+                                       ref,
+                                       label,
+                                       labelPath);
   }
 
   void MapPainterAgg::DrawLabels(const Projection& projection,
@@ -443,7 +454,7 @@ namespace osmscout {
                                       const MapParameter& parameter,
                                       const MapData& /*data*/)
   {
-    labelLayouter.SetViewport(DoubleRectangle(0, 0, projection.GetWidth(), projection.GetHeight()));
+    labelLayouter.SetViewport(ScreenVectorRectangle(0, 0, projection.GetWidth(), projection.GetHeight()));
     labelLayouter.SetLayoutOverlap(projection.ConvertWidthToPixel(parameter.GetLabelLayouterOverlap()));
   }
 
@@ -456,7 +467,7 @@ namespace osmscout {
   }
 
   void MapPainterAgg::DrawIcon(const IconStyle* /*style*/,
-                               double /*centerX*/, double /*centerY*/,
+                               const Vertex2D& /*centerPos*/,
                                double /*width*/, double /*height*/)
   {
     // Not implemented
@@ -465,7 +476,7 @@ namespace osmscout {
   void MapPainterAgg::DrawSymbol(const Projection& projection,
                                  const MapParameter& parameter,
                                  const Symbol& symbol,
-                                 double x, double y,
+                                 const Vertex2D& screenPos,
                                  double /*scaleFactor*/)
   {
     ScreenBox boundingBox=symbol.GetBoundingBox(projection);
@@ -485,12 +496,12 @@ namespace osmscout {
              pixel!=polygon->GetCoords().end();
              ++pixel) {
           if (pixel==polygon->GetCoords().begin()) {
-            path.move_to(x+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
-                         y+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
+            path.move_to(screenPos.GetX()+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
+                         screenPos.GetY()+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
           }
           else {
-            path.line_to(x+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
-                         y+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
+            path.line_to(screenPos.GetX()+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
+                         screenPos.GetY()+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
           }
         }
 
@@ -510,8 +521,8 @@ namespace osmscout {
         FillStyleRef      fillStyle=rectangle->GetFillStyle();
         BorderStyleRef    borderStyle=rectangle->GetBorderStyle();
         agg::path_storage path;
-        double            xPos=x+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetX())-center.GetX();
-        double            yPos=y+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetY())-center.GetY();
+        double            xPos=screenPos.GetX()+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetX())-center.GetX();
+        double            yPos=screenPos.GetY()+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetY())-center.GetY();
         double            width=projection.ConvertWidthToPixel(rectangle->GetWidth());
         double            height=projection.ConvertWidthToPixel(rectangle->GetHeight());
 
@@ -538,8 +549,8 @@ namespace osmscout {
         agg::path_storage path;
         double            radius=projection.ConvertWidthToPixel(circle->GetRadius());
 
-        agg::ellipse ellipse(x+projection.ConvertWidthToPixel(circle->GetCenter().GetX())-center.GetX(),
-                             y+projection.ConvertWidthToPixel(circle->GetCenter().GetY())-center.GetY(),
+        agg::ellipse ellipse(screenPos.GetX()+projection.ConvertWidthToPixel(circle->GetCenter().GetX())-center.GetX(),
+                             screenPos.GetY()+projection.ConvertWidthToPixel(circle->GetCenter().GetY())-center.GetY(),
                              radius,
                              radius);
 
