@@ -654,7 +654,7 @@ namespace osmscout {
                                       const MapParameter& parameter,
                                       const MapData& /*data*/)
   {
-    DoubleRectangle viewport;
+    ScreenVectorRectangle viewport;
     double x2, y2;
     cairo_clip_extents(draw, &viewport.x, &viewport.y, &x2, &y2);
     viewport.width = x2 - viewport.x;
@@ -714,8 +714,8 @@ namespace osmscout {
         result.back().glyph.font = font;
 
 
-        result.back().position.SetX(((double)glyphInfo.geometry.x_offset/(double)PANGO_SCALE) + horizontalOffset);
-        result.back().position.SetY((double)glyphInfo.geometry.y_offset/(double)PANGO_SCALE);
+        result.back().position=Vertex2D(((double)glyphInfo.geometry.x_offset/(double)PANGO_SCALE) + horizontalOffset,
+                                        (double)glyphInfo.geometry.y_offset/(double)PANGO_SCALE);
 
         if constexpr (debugLabelLayouter) {
           std::cout << "     " << glyphInfo.glyph << ": " << result.back().position.GetX() << " x "
@@ -787,16 +787,16 @@ namespace osmscout {
     return label;
   }
 
-  DoubleRectangle MapPainterCairo::GlyphBoundingBox(const CairoNativeGlyph &glyph) const
+  ScreenVectorRectangle MapPainterCairo::GlyphBoundingBox(const CairoNativeGlyph &glyph) const
   {
     assert(glyph.glyphString->num_glyphs == 1);
     PangoRectangle extends;
     pango_font_get_glyph_extents(glyph.font.get(), glyph.glyphString->glyphs[0].glyph, nullptr, &extends);
 
-    return DoubleRectangle((double)(extends.x) / (double)PANGO_SCALE,
-                           (double)(extends.y) / (double)PANGO_SCALE,
-                           (double)(extends.width) / (double)PANGO_SCALE,
-                           (double)(extends.height) / (double)PANGO_SCALE);
+    return ScreenVectorRectangle((double)(extends.x) / (double)PANGO_SCALE,
+                                 (double)(extends.y) / (double)PANGO_SCALE,
+                                 (double)(extends.width) / (double)PANGO_SCALE,
+                                 (double)(extends.height) / (double)PANGO_SCALE);
   }
 
   void MapPainterCairo::DrawGlyphs(const Projection &/*projection*/,
@@ -848,12 +848,12 @@ namespace osmscout {
 
 #else
 
-  DoubleRectangle MapPainterCairo::GlyphBoundingBox(const CairoNativeGlyph &glyph) const
+  ScreenVectorRectangle MapPainterCairo::GlyphBoundingBox(const CairoNativeGlyph &glyph) const
   {
-    return DoubleRectangle(0,
-                           glyph.height * -1,
-                           glyph.width,
-                           glyph.height);
+    return ScreenVectorRectangle(0,
+                                 glyph.height * -1,
+                                 glyph.width,
+                                 glyph.height);
   }
 
   template<>
@@ -875,8 +875,7 @@ namespace osmscout {
       result.back().glyph.width = textExtents.width;
       result.back().glyph.height = label.fontExtents.height;
 
-      result.back().position.SetX(horizontalOffset);
-      result.back().position.SetY(0);
+      result.back().position=Vertex2D(horizontalOffset,0);
 
       horizontalOffset += result.back().glyph.width;
     }
@@ -944,7 +943,7 @@ namespace osmscout {
 
   void MapPainterCairo::DrawLabel(const Projection &/*projection*/,
                                   const MapParameter &/*parameter*/,
-                                  const DoubleRectangle &labelRectangle,
+                                  const ScreenVectorRectangle &labelRectangle,
                                   const LabelData &label,
                                   const CairoNativeLabel &layout)
   {
@@ -1079,12 +1078,14 @@ namespace osmscout {
 
   void MapPainterCairo::RegisterRegularLabel(const Projection &projection,
                                              const MapParameter &parameter,
+                                             const ObjectFileRef& ref,
                                              const std::vector<LabelData> &labels,
                                              const Vertex2D &position,
                                              double objectWidth)
   {
     labelLayouter.RegisterLabel(projection,
                                 parameter,
+                                ref,
                                 position,
                                 labels,
                                 objectWidth);
@@ -1094,12 +1095,14 @@ namespace osmscout {
    * Register contour label
    */
   void MapPainterCairo::RegisterContourLabel(const Projection &projection,
-                                    const MapParameter &parameter,
-                                    const PathLabelData &label,
-                                    const LabelPath &labelPath)
+                                             const MapParameter &parameter,
+                                             const ObjectFileRef& ref,
+                                             const PathLabelData &label,
+                                             const LabelPath &labelPath)
   {
     labelLayouter.RegisterContourLabel(projection,
                                        parameter,
+                                       ref,
                                        label,
                                        labelPath);
   }
@@ -1120,19 +1123,19 @@ namespace osmscout {
   void MapPainterCairo::DrawSymbol(const Projection& projection,
                                    const MapParameter& /*parameter*/,
                                    const Symbol& symbol,
-                                   double x, double y,
+                                   const Vertex2D& screenPos,
                                    double scaleFactor)
   {
     SymbolRendererCairo renderer(draw);
 
     renderer.Render(projection,
                     symbol,
-                    Vertex2D(x, y),
+                    screenPos,
                     scaleFactor);
   }
 
   void MapPainterCairo::DrawIcon(const IconStyle* style,
-                                 double centerX, double centerY,
+                                 const Vertex2D& centerPos,
                                  double width, double height)
   {
     size_t idx=style->GetIconId()-1;
@@ -1152,8 +1155,8 @@ namespace osmscout {
 
     cairo_set_source_surface(draw,
                              icon,
-                             (centerX-width/2) / scaleW,
-                             (centerY-height/2) / scaleH);
+                             (centerPos.GetX()-width/2) / scaleW,
+                             (centerPos.GetY()-height/2) / scaleH);
 
     cairo_paint(draw);
     cairo_set_matrix(draw, &matrix);
