@@ -18,6 +18,7 @@
 */
 
 #import <osmscoutmapiosx/MapPainterIOS.h>
+#import <osmscoutmapiosx/SymbolRendererIOS.h>
 
 #include <cassert>
 #include <limits>
@@ -293,10 +294,10 @@ namespace osmscout {
         return averageCharWidth.insert(std::pair<size_t,double>(convFontSize,averageWidth)).first->second;
     }
 
-    DoubleRectangle MapPainterIOS::GlyphBoundingBox(const IOSGlyphInRun &glyph) const {
+    ScreenVectorRectangle MapPainterIOS::GlyphBoundingBox(const IOSGlyphInRun &glyph) const {
         CFRange range = CFRangeMake(glyph.index, 1);
         CGRect glyphRect = CTRunGetImageBounds(glyph.run, cg, range);
-        return DoubleRectangle(glyphRect.origin.x,glyphRect.origin.y,glyphRect.size.width,glyphRect.size.height);
+        return ScreenVectorRectangle(glyphRect.origin.x,glyphRect.origin.y,glyphRect.size.width,glyphRect.size.height);
     }
 
     template<> std::vector<IOSGlyph> IOSLabel::ToGlyphs() const {
@@ -313,7 +314,7 @@ namespace osmscout {
                 glyph.glyph.line = label.line[i];
                 glyph.glyph.run = run;
                 glyph.glyph.index = index;
-                glyph.position.Set(glyphPositions[index].x, glyphPositions[index].y);
+                glyph.position=Vertex2D(glyphPositions[index].x, glyphPositions[index].y);
                 result.push_back(std::move(glyph));
             }
             i++;
@@ -377,11 +378,11 @@ namespace osmscout {
             log.Debug() << "proposedWidth=" << proposedWidth << " for '" << text << "'";
         }
 
-        
+
         NSString *str = [NSString stringWithCString:text.c_str() encoding:NSUTF8StringEncoding];
-        // replace hyphen by unicode NON-BREAKING HYPHEN  
+        // replace hyphen by unicode NON-BREAKING HYPHEN
         str = [str stringByReplacingOccurrencesOfString:@"-" withString:@"\u2011"];
-        
+
         Font *font = GetFont(projection, parameter, fontSize);
         NSMutableDictionary<NSAttributedStringKey, id> *attr = [NSMutableDictionary dictionaryWithDictionary: @{}];
         if(font){
@@ -414,9 +415,9 @@ namespace osmscout {
             }
             result->label.lineWidth = rect.size.width;
             result->label.lineHeight = GetFontHeight(projection, parameter, fontSize);
-            
+
             log.Debug() << "Layout '"<<text<<"' width=" << rect.size.width <<" height=" << rect.size.height;
-            
+
             result->text = text;
             result->fontSize = fontSize;
             result->width = rect.size.width;
@@ -444,7 +445,7 @@ namespace osmscout {
         int lineNumber = 0;
         for (CTRunRef run : layout.run) {
             const CTFontRef font = (CTFontRef)CFDictionaryGetValue(CTRunGetAttributes(run), kCTFontAttributeName);
-            CFIndex glyphCount = CTRunGetGlyphCount(run);
+            const CFIndex glyphCount = CTRunGetGlyphCount(run);
             CGGlyph glyphs[glyphCount];
             CGPoint glyphPositions[glyphCount];
             CGSize glyphAdvances[glyphCount];
@@ -486,7 +487,7 @@ namespace osmscout {
 
     void MapPainterIOS::DrawLabel(const Projection& /*projection*/,
                                   const MapParameter& /*parameter*/,
-                                  const DoubleRectangle& labelRect,
+                                  const ScreenVectorRectangle& labelRect,
                                   const LabelData& label,
                                   const IOSRunInLine& layout) {
 
@@ -538,23 +539,25 @@ namespace osmscout {
                                       const Projection& projection,
                                       const MapParameter& parameter,
                                       const MapData& /*data*/){
-        labelLayouter.SetViewport(DoubleRectangle(0, 0, CGBitmapContextGetWidth(cg), CGBitmapContextGetHeight(cg)));
+        labelLayouter.SetViewport(ScreenVectorRectangle(0, 0, CGBitmapContextGetWidth(cg), CGBitmapContextGetHeight(cg)));
         labelLayouter.SetLayoutOverlap(projection.ConvertWidthToPixel(parameter.GetLabelLayouterOverlap()));
     }
 
     void MapPainterIOS::RegisterRegularLabel(const Projection &projection,
                                              const MapParameter &parameter,
+                                             const ObjectFileRef& ref,
                                              const std::vector<LabelData> &labels,
                                              const Vertex2D &position,
                                              double objectWidth){
-        labelLayouter.RegisterLabel(projection, parameter, position, labels, objectWidth);
+        labelLayouter.RegisterLabel(projection, parameter, ref, position, labels, objectWidth);
     }
 
     void MapPainterIOS::RegisterContourLabel(const Projection &projection,
                                       const MapParameter &parameter,
+                                      const ObjectFileRef& ref,
                                       const PathLabelData &label,
                                       const LabelPath &labelPath){
-        labelLayouter.RegisterContourLabel(projection, parameter, label, labelPath);
+        labelLayouter.RegisterContourLabel(projection, parameter, ref, label, labelPath);
     }
 
     void MapPainterIOS::DrawLabels(const Projection& projection,
@@ -586,7 +589,7 @@ namespace osmscout {
             hnd.transEnd = transStart;
         }
         hnd.direction = (hnd.transStart < hnd.transEnd) ? 1 : -1;
-        origin.Set(coordBuffer.buffer[hnd.transStart].GetX(), coordBuffer.buffer[hnd.transStart].GetY());
+        origin=Vertex2D(coordBuffer.buffer[hnd.transStart].GetX(), coordBuffer.buffer[hnd.transStart].GetY());
     }
 
     bool MapPainterIOS::followPath(FollowPathHandle &hnd, double l, Vertex2D &origin) {
@@ -609,7 +612,7 @@ namespace osmscout {
 
             fracToGo = l/len;
             if(fracToGo <= 1.0) {
-                origin.Set(x + deltaX*fracToGo,y + deltaY*fracToGo);
+                origin=Vertex2D(x + deltaX*fracToGo,y + deltaY*fracToGo);
                 return true;
             }
 
@@ -623,13 +626,12 @@ namespace osmscout {
     }
 
     void MapPainterIOS::DrawContourSymbol(const Projection& projection,
-                                          const MapParameter& parameter,
+                                          const MapParameter& /* parameter */,
                                           const Symbol& symbol,
                                           const ContourSymbolData& data){
-
+        SymbolRendererIOS renderer(cg);
         ScreenBox boundingBox=symbol.GetBoundingBox(projection);
         double width=boundingBox.GetWidth();
-        double height=boundingBox.GetHeight();
         bool isClosed = false;
         CGAffineTransform transform=CGAffineTransformMake(1.0, 0.0, 0.0, 1.0, 0.0, 0.0);
         Vertex2D origin;
@@ -640,6 +642,7 @@ namespace osmscout {
         if(!isClosed && !followPath(followPathHnd, data.symbolOffset, origin)){
             return;
         }
+
         bool loop = true;
         while (loop){
             x1 = origin.GetX();
@@ -657,7 +660,14 @@ namespace osmscout {
                     CGContextTranslateCTM(cg, x2, y2);
                     CGAffineTransform ct = CGAffineTransformConcat(transform, CGAffineTransformMakeRotation(slope));
                     CGContextConcatCTM(cg, ct);
-                    DrawSymbol(projection, parameter, symbol, 0, 0, data.symbolScale);
+                    renderer.Render(projection,
+                                    symbol,
+                                    Vertex2D(0.0, 0.0),
+                                    []() {
+                                    },
+                                    []() {
+                                    },
+                                    data.symbolScale);
                     CGContextRestoreGState(cg);
                     loop = followPath(followPathHnd, data.symbolSpace, origin);
                 }
@@ -672,8 +682,8 @@ namespace osmscout {
      *          double width, double height)
      */
     void MapPainterIOS::DrawIcon(const IconStyle* style,
-                                 double x, double y,
-                                 double /*width*/, double /*height*/){
+                                 const Vertex2D& centerPos,
+                                 double /*width*/, double /*height*/) {
         size_t idx=style->GetIconId()-1;
 
         assert(idx<images.size());
@@ -681,13 +691,16 @@ namespace osmscout {
 
         CGFloat w = CGImageGetWidth(images[idx]);
         CGFloat h = CGImageGetHeight(images[idx]);
-        CGRect rect = CGRectMake(x-w/2, -h/2-y, w, h);
+        CGRect rect = CGRectMake(centerPos.GetX()-w/2,
+                                 -h/2-centerPos.GetY(),
+                                 w, h);
         CGContextSaveGState(cg);
         CGContextScaleCTM(cg, 1.0, -1.0);
         CGContextDrawImage(cg, rect, images[idx]);
         CGContextRestoreGState(cg);
     }
 
+
     /*
      * DrawSymbol(const Projection& projection,
      *            const MapParameter& parameter,
@@ -695,102 +708,17 @@ namespace osmscout {
      *            double x, double y, double scaleFactor)
      */
     void MapPainterIOS::DrawSymbol(const Projection& projection,
-                    const MapParameter& parameter,
+                    const MapParameter& /* parameter */,
                     const Symbol& symbol,
-                    double x, double y,
-                    double scaleFactor){
-        ScreenBox boundingBox=symbol.GetBoundingBox(projection);
-        Vertex2D center=boundingBox.GetCenter();
+                    const Vertex2D& screenPos,
+                    double scaleFactor) {
 
-        CGContextSaveGState(cg);
-        if (scaleFactor != 0.0) {
-            CGContextConcatCTM(cg, CGAffineTransformMakeScale(scaleFactor, scaleFactor));
-        }
-        for (const auto& primitive : symbol.GetPrimitives()) {
-            const DrawPrimitive *primitivePtr=primitive.get();
+        SymbolRendererIOS renderer(cg);
 
-            if (const auto *polygon = dynamic_cast<const PolygonPrimitive*>(primitivePtr);
-                polygon != nullptr) {
-
-                FillStyleRef fillStyle=polygon->GetFillStyle();
-                BorderStyleRef borderStyle=polygon->GetBorderStyle();
-
-                if (fillStyle) {
-                    SetFill(projection, parameter, *fillStyle);
-                } else {
-                    CGContextSetRGBFillColor(cg,0,0,0,0);
-                }
-
-                if (borderStyle) {
-                    SetBorder(projection, parameter, *borderStyle);
-                } else {
-                    CGContextSetRGBStrokeColor(cg,0,0,0,0);
-                }
-
-                CGContextBeginPath(cg);
-
-                for (std::list<Vertex2D>::const_iterator pixel=polygon->GetCoords().begin();
-                     pixel!=polygon->GetCoords().end();
-                     ++pixel) {
-                    if (pixel==polygon->GetCoords().begin()) {
-                        CGContextMoveToPoint(cg,x+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
-                                             y+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
-                    } else {
-                        CGContextAddLineToPoint(cg,x+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX(),
-                                                y+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
-                    }
-                }
-
-                CGContextDrawPath(cg, kCGPathFillStroke);
-            }
-            else if (const auto *rectangle = dynamic_cast<const RectanglePrimitive*>(primitivePtr);
-                     rectangle != nullptr) {
-
-                FillStyleRef fillStyle=rectangle->GetFillStyle();
-                BorderStyleRef borderStyle=rectangle->GetBorderStyle();
-                if (fillStyle) {
-                    SetFill(projection, parameter, *fillStyle);
-                } else {
-                    CGContextSetRGBFillColor(cg,0,0,0,0);
-                }
-
-                if (borderStyle) {
-                    SetBorder(projection, parameter, *borderStyle);
-                } else {
-                    CGContextSetRGBStrokeColor(cg,0,0,0,0);
-                }
-                CGRect rect = CGRectMake(x+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetX())-center.GetX(),
-                                         y+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetY())-center.GetY(),
-                                         projection.ConvertWidthToPixel(rectangle->GetWidth()),
-                                         projection.ConvertWidthToPixel(rectangle->GetHeight()));
-                CGContextAddRect(cg,rect);
-                CGContextDrawPath(cg, kCGPathFillStroke);
-            }
-            else if (const auto *circle = dynamic_cast<const CirclePrimitive*>(primitivePtr);
-                     circle != nullptr) {
-
-                FillStyleRef fillStyle=circle->GetFillStyle();
-                BorderStyleRef borderStyle=circle->GetBorderStyle();
-                if (fillStyle) {
-                    SetFill(projection, parameter, *fillStyle);
-                } else {
-                    CGContextSetRGBFillColor(cg,0,0,0,0);
-                }
-
-                if (borderStyle) {
-                    SetBorder(projection, parameter, *borderStyle);
-                } else {
-                    CGContextSetRGBStrokeColor(cg,0,0,0,0);
-                }
-                CGRect rect = CGRectMake(x+projection.ConvertWidthToPixel(circle->GetCenter().GetX())-center.GetX(),
-                                         y+projection.ConvertWidthToPixel(circle->GetCenter().GetY())-center.GetY(),
-                                         projection.ConvertWidthToPixel(circle->GetRadius()),
-                                         projection.ConvertWidthToPixel(circle->GetRadius()));
-                CGContextAddEllipseInRect(cg, rect);
-                CGContextDrawPath(cg, kCGPathFillStroke);
-            }
-        }
-        CGContextRestoreGState(cg);
+        renderer.Render(projection,
+                        symbol,
+                        screenPos,
+                        scaleFactor);
     }
 
     /*
@@ -883,33 +811,6 @@ namespace osmscout {
         }
     }
 
-    /*
-     * SetPen(const LineStyle& style,
-     *        double lineWidth)
-     */
-    void MapPainterIOS::SetPen(const LineStyle& style,
-                              double lineWidth) {
-        CGContextSetRGBStrokeColor(cg,style.GetLineColor().GetR(),
-                                      style.GetLineColor().GetG(),
-                                      style.GetLineColor().GetB(),
-                                      style.GetLineColor().GetA());
-        CGContextSetLineWidth(cg,lineWidth);
-
-        if (style.GetDash().empty()) {
-            CGContextSetLineDash(cg, 0.0, NULL, 0);
-            CGContextSetLineCap(cg, kCGLineCapRound);
-        } else {
-            CGFloat *dashes = (CGFloat *)malloc(sizeof(CGFloat)*style.GetDash().size());
-            for (size_t i=0; i<style.GetDash().size(); i++) {
-                dashes[i] = style.GetDash()[i]*lineWidth;
-            }
-            CGContextSetLineDash(cg, 0.0, dashes, style.GetDash().size());
-            free(dashes); dashes = NULL;
-            CGContextSetLineCap(cg, kCGLineCapButt);
-        }
-    }
-
-
     /*
      * DrawArea(const Projection& projection,
      *          const MapParameter& parameter,
