@@ -31,9 +31,11 @@
 #include <osmscout/system/Assert.h>
 #include <osmscout/system/Math.h>
 
-#include <osmscout/util/File.h>
+#include <osmscout/io/File.h>
+
 #include <osmscout/util/Geometry.h>
-#include <osmscout/util/Logger.h>
+#include <osmscout/log/Logger.h>
+
 #include <osmscoutmapqt/SymbolRendererQt.h>
 
 namespace osmscout {
@@ -300,7 +302,7 @@ namespace osmscout {
 
   void MapPainterQt::DrawLabel(const Projection& /*projection*/,
                                const MapParameter& /*parameter*/,
-                               const DoubleRectangle& labelRect,
+                               const ScreenVectorRectangle& labelRect,
                                const LabelData& label,
                                const QTextLayout& textLayout)
   {
@@ -523,8 +525,7 @@ namespace osmscout {
     }
 
     hnd.direction=(hnd.transStart < hnd.transEnd) ? 1 : -1;
-    origin.Set(coordRange.Get(hnd.transStart).GetX(),
-               coordRange.Get(hnd.transStart).GetY());
+    origin=coordRange.Get(hnd.transStart);
   }
 
   bool MapPainterQt::FollowPath(FollowPathHandle &hnd,
@@ -553,8 +554,8 @@ namespace osmscout {
       double fracToGo=l/len;
 
       if (fracToGo<=1.0) {
-        origin.Set(x + deltaX*fracToGo,
-                   y + deltaY*fracToGo);
+        origin=Vertex2D(x + deltaX*fracToGo,
+                        y + deltaY*fracToGo);
         return true;
       }
 
@@ -634,7 +635,7 @@ namespace osmscout {
           DrawSymbol(projection,
                      parameter,
                      symbol,
-                     0,0,
+                     Vertex2D::ZERO,
                      data.symbolScale);
           loop=FollowPath(followPathHnd,
                           data.coordRange,
@@ -647,7 +648,7 @@ namespace osmscout {
   }
 
   void MapPainterQt::DrawIcon(const IconStyle* style,
-                              double x, double y,
+                              const Vertex2D& centerPos,
                               double width, double height)
   {
     const auto &it=images.find(style->GetIconName());
@@ -655,7 +656,8 @@ namespace osmscout {
     assert(it!=images.end());
     assert(!it->second.isNull());
 
-    painter->drawImage(QRectF(x-width/2, y-height/2, width, height),
+    painter->drawImage(QRectF(centerPos.GetX()-width/2, centerPos.GetY()-height/2,
+                              width, height),
                        it->second,
                        QRectF(0, 0, it->second.width(), it->second.height()));
   }
@@ -663,13 +665,13 @@ namespace osmscout {
   void MapPainterQt::DrawSymbol(const Projection& projection,
                                 const MapParameter& /*parameter*/,
                                 const Symbol& symbol,
-                                double x, double y,
+                                const Vertex2D& screenPos,
                                 double scaleFactor)
   {
     SymbolRendererQt renderer(painter);
     renderer.Render(projection,
                     symbol,
-                    Vertex2D(x, y),
+                    screenPos,
                     scaleFactor);
   }
 
@@ -842,7 +844,7 @@ namespace osmscout {
                                    const MapParameter& parameter,
                                    const MapData& /*data*/)
   {
-    labelLayouter.SetViewport(DoubleRectangle(0, 0, painter->window().width(), painter->window().height()));
+    labelLayouter.SetViewport(ScreenVectorRectangle(0, 0, painter->window().width(), painter->window().height()));
     labelLayouter.SetLayoutOverlap(projection.ConvertWidthToPixel(parameter.GetLabelLayouterOverlap()));
   }
 
@@ -869,19 +871,30 @@ namespace osmscout {
 
   void MapPainterQt::RegisterRegularLabel(const Projection &projection,
                                           const MapParameter &parameter,
+                                          const ObjectFileRef& ref,
                                           const std::vector<LabelData> &labels,
                                           const Vertex2D &position,
                                           double objectWidth)
   {
-    GetLayouter().RegisterLabel(projection, parameter, position, labels, objectWidth);
+    GetLayouter().RegisterLabel(projection,
+                                parameter,
+                                ref,
+                                position,
+                                labels,
+                                objectWidth);
   }
 
   void MapPainterQt::RegisterContourLabel(const Projection &projection,
                                           const MapParameter &parameter,
+                                          const ObjectFileRef& ref,
                                           const PathLabelData &label,
                                           const LabelPath &labelPath)
   {
-    GetLayouter().RegisterContourLabel(projection, parameter, label, labelPath);
+    GetLayouter().RegisterContourLabel(projection,
+                                       parameter,
+                                       ref,
+                                       label,
+                                       labelPath);
   }
 
   void MapPainterQt::DrawLabels(const Projection& projection,
@@ -901,11 +914,11 @@ namespace osmscout {
     labelLayouter.Reset();
   }
 
-  DoubleRectangle MapPainterQt::GlyphBoundingBox(const QGlyphRun &glyph) const
+  ScreenVectorRectangle MapPainterQt::GlyphBoundingBox(const QGlyphRun &glyph) const
   {
     auto bbox=glyph.boundingRect();
     auto tl=bbox.topLeft();
-    return DoubleRectangle(tl.x(), tl.y(), bbox.width(), bbox.height());
+    return ScreenVectorRectangle(tl.x(), tl.y(), bbox.width(), bbox.height());
   }
 
   void MapPainterQt::DrawGlyph(QPainter *painter, const Glyph<QGlyphRun> &glyph) const
@@ -1073,19 +1086,19 @@ namespace osmscout {
 
         QtGlyph glyph;
         glyph.glyph=std::move(orphanGlyph);
-        glyph.position.Set(pos.x(), pos.y());
+        glyph.position=Vertex2D(pos.x(), pos.y());
         result.push_back(std::move(glyph));
       }
     }
     return result;
   }
 
-  MapPainterBatchQt::MapPainterBatchQt(size_t expectedCount):
-    MapPainterBatch(expectedCount) {}
+  BatchMapPainterQt::BatchMapPainterQt(size_t expectedCount):
+    BatchMapPainter(expectedCount) {}
 
-  MapPainterBatchQt::~MapPainterBatchQt(){}
+  BatchMapPainterQt::~BatchMapPainterQt(){}
 
-  bool MapPainterBatchQt::paint(const Projection& projection,
+  bool BatchMapPainterQt::paint(const Projection& projection,
                                 const MapParameter& parameter,
                                 QPainter* qPainter)
   {
