@@ -26,8 +26,10 @@
 
 #include <osmscout/system/Assert.h>
 #include <osmscout/system/Math.h>
+
+#include <osmscout/io/File.h>
+
 #include <osmscout/util/String.h>
-#include <osmscout/util/File.h>
 #include <osmscout/util/Base64.h>
 
 namespace osmscout {
@@ -135,13 +137,13 @@ namespace osmscout {
     return label;
   }
 
-  DoubleRectangle MapPainterSVG::GlyphBoundingBox(const NativeGlyph &glyph) const
+  ScreenVectorRectangle MapPainterSVG::GlyphBoundingBox(const NativeGlyph &glyph) const
   {
     assert(glyph.glyphString->num_glyphs == 1);
     PangoRectangle extends;
     pango_font_get_glyph_extents(glyph.font.get(), glyph.glyphString->glyphs[0].glyph, nullptr, &extends);
 
-    return DoubleRectangle((double)(extends.x) / (double)PANGO_SCALE,
+    return ScreenVectorRectangle((double)(extends.x) / (double)PANGO_SCALE,
                            (double)(extends.y) / (double)PANGO_SCALE,
                            (double)(extends.width) / (double)PANGO_SCALE,
                            (double)(extends.height) / (double)PANGO_SCALE);
@@ -198,8 +200,8 @@ namespace osmscout {
 
         result.back().glyph.font = font;
 
-        result.back().position.SetX(((double)glyphInfo.geometry.x_offset/(double)PANGO_SCALE) + horizontalOffset);
-        result.back().position.SetY((double)glyphInfo.geometry.y_offset/(double)PANGO_SCALE);
+        result.back().position=Vertex2D(((double)glyphInfo.geometry.x_offset/(double)PANGO_SCALE) + horizontalOffset,
+                                        (double)glyphInfo.geometry.y_offset/(double)PANGO_SCALE);
 
         glyphInfo.geometry.x_offset = 0;
         glyphInfo.geometry.y_offset = 0;
@@ -236,20 +238,20 @@ namespace osmscout {
 
       result.back().glyph.character = WStringToUTF8String(label.substr(ch,1));
 
-      result.back().position.SetX(horizontalOffset);
-      result.back().position.SetY(0);
+      result.back().position=Vertex2D(horizontalOffset,
+                                      0.0);
 
       horizontalOffset += (double)(height * MapPainterSVG::AverageCharacterWidth);
     }
     return result;
   }
 
-  DoubleRectangle MapPainterSVG::GlyphBoundingBox(const NativeGlyph &glyph) const
+  ScreenVectorRectangle MapPainterSVG::GlyphBoundingBox(const NativeGlyph &glyph) const
   {
-    return DoubleRectangle(0.0,
-                           (double)(glyph.height * -1),
-                           (double)glyph.width,
-                           (double)glyph.height);
+    return ScreenVectorRectangle(0.0,
+                                 (double)(glyph.height * -1),
+                                 (double)glyph.width,
+                                 (double)glyph.height);
   }
 
   std::shared_ptr<MapPainterSVG::SvgLabel> MapPainterSVG::Layout(const Projection& projection,
@@ -273,7 +275,7 @@ namespace osmscout {
 
   void MapPainterSVG::DrawLabel(const Projection &projection,
                                 const MapParameter &parameter,
-                                const DoubleRectangle &labelRectangle,
+                                const ScreenVectorRectangle &labelRectangle,
                                 const LabelData &label,
                                 const NativeLabel &/*layout*/)
   {
@@ -602,8 +604,8 @@ namespace osmscout {
   {
     stream << "  <g id=\"map\">" << std::endl;
 
-    DoubleRectangle viewport(0.0,0.0,
-                             (double)projection.GetWidth(), (double)projection.GetHeight());
+    ScreenVectorRectangle viewport(0.0,0.0,
+                                   (double)projection.GetWidth(), (double)projection.GetHeight());
 
     labelLayouter.SetViewport(viewport);
     labelLayouter.SetLayoutOverlap(projection.ConvertWidthToPixel(parameter.GetLabelLayouterOverlap()));
@@ -716,11 +718,17 @@ namespace osmscout {
 
   void MapPainterSVG::RegisterRegularLabel(const Projection &projection,
                                            const MapParameter &parameter,
+                                           const ObjectFileRef& ref,
                                            const std::vector<LabelData> &labels,
                                            const Vertex2D &position,
                                            double objectWidth)
   {
-    labelLayouter.RegisterLabel(projection, parameter, position, labels, objectWidth);
+    labelLayouter.RegisterLabel(projection,
+                                parameter,
+                                ref,
+                                position,
+                                labels,
+                                objectWidth);
   }
 
   /**
@@ -728,10 +736,15 @@ namespace osmscout {
    */
   void MapPainterSVG::RegisterContourLabel(const Projection &projection,
                                            const MapParameter &parameter,
+                                           const ObjectFileRef& ref,
                                            const PathLabelData &label,
                                            const LabelPath &labelPath)
   {
-    labelLayouter.RegisterContourLabel(projection, parameter, label, labelPath);
+    labelLayouter.RegisterContourLabel(projection,
+                                       parameter,
+                                       ref,
+                                       label,
+                                       labelPath);
   }
 
   void MapPainterSVG::IconData(const Projection& projection,
@@ -777,7 +790,7 @@ namespace osmscout {
   void MapPainterSVG::DrawSymbol(const Projection& projection,
                                  const MapParameter& /*parameter*/,
                                  const Symbol& symbol,
-                                 double x, double y,
+                                 const Vertex2D& screenPos,
                                  double /*scaleFactor*/)
   {
     ScreenBox boundingBox=symbol.GetBoundingBox(projection);
@@ -792,8 +805,8 @@ namespace osmscout {
       const auto *circle=dynamic_cast<const CirclePrimitive*>(primitivePtr);
 
       if (polygon != nullptr) {
-        FillStyleRef   fillStyle=polygon->GetFillStyle();
-        BorderStyleRef borderStyle=polygon->GetBorderStyle();
+        const FillStyleRef   fillStyle=polygon->GetFillStyle();
+        const BorderStyleRef borderStyle=polygon->GetBorderStyle();
 
         stream << "    <polyline";
         SetupFillAndStroke(fillStyle, borderStyle);
@@ -808,19 +821,19 @@ namespace osmscout {
             stream << " ";
           }
 
-          stream << (x+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX())
-                 << "," << (y+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
+          stream << (screenPos.GetX()+projection.ConvertWidthToPixel(pixel->GetX())-center.GetX())
+                 << "," << (screenPos.GetY()+projection.ConvertWidthToPixel(pixel->GetY())-center.GetY());
         }
 
         stream << "\" />" << std::endl;
 
       } else if (rectangle != nullptr) {
-        FillStyleRef   fillStyle=rectangle->GetFillStyle();
-        BorderStyleRef borderStyle=rectangle->GetBorderStyle();
+        const FillStyleRef   fillStyle=rectangle->GetFillStyle();
+        const BorderStyleRef borderStyle=rectangle->GetBorderStyle();
 
         stream << "    <rect";
-        stream << " x=\"" << ((x+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetX())-center.GetX())) << "\"";
-        stream << " y=\"" << ((y+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetY())-center.GetY())) << "\"";
+        stream << " x=\"" << ((screenPos.GetX()+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetX())-center.GetX())) << "\"";
+        stream << " y=\"" << ((screenPos.GetY()+projection.ConvertWidthToPixel(rectangle->GetTopLeft().GetY())-center.GetY())) << "\"";
         stream << " width=\"" << (projection.ConvertWidthToPixel(rectangle->GetWidth())) << "\"";
         stream << " height=\"" << (projection.ConvertWidthToPixel(rectangle->GetHeight())) << "\"";
 
@@ -828,12 +841,12 @@ namespace osmscout {
         stream << " />" << std::endl;
 
       } else if (circle != nullptr) {
-        FillStyleRef   fillStyle=circle->GetFillStyle();
-        BorderStyleRef borderStyle=circle->GetBorderStyle();
+        const FillStyleRef   fillStyle=circle->GetFillStyle();
+        const BorderStyleRef borderStyle=circle->GetBorderStyle();
 
         stream << "    <circle";
-        stream << " cx=\"" << (x+projection.ConvertWidthToPixel(circle->GetCenter().GetX())-center.GetX()) << "\"";
-        stream << " cy=\"" << (y+projection.ConvertWidthToPixel(circle->GetCenter().GetY())-center.GetY()) << "\"";
+        stream << " cx=\"" << (screenPos.GetX()+projection.ConvertWidthToPixel(circle->GetCenter().GetX())-center.GetX()) << "\"";
+        stream << " cy=\"" << (screenPos.GetY()+projection.ConvertWidthToPixel(circle->GetCenter().GetY())-center.GetY()) << "\"";
         stream << " r=\"" << (projection.ConvertWidthToPixel(circle->GetRadius())) << "\"";
 
         SetupFillAndStroke(fillStyle, borderStyle);
@@ -843,10 +856,10 @@ namespace osmscout {
   }
 
   void MapPainterSVG::DrawIcon(const IconStyle* style,
-                               double x, double y,
+                               const Vertex2D& centerPos,
                                double width, double height)
   {
-    stream << "    <use x=\"" << (x - width/2) << "\" y=\"" << (y - height/2) << "\" " <<
+    stream << "    <use x=\"" << (centerPos.GetX() - width/2) << "\" y=\"" << (centerPos.GetY() - height/2) << "\" " <<
       "width=\"" << width << "\" height=\"" << height << "\" " <<
       "xlink:href=\"#icon_" << style->GetIconId() << "\" />\n";
   }
