@@ -21,18 +21,51 @@
 
 #include <algorithm>
 
-#include <osmscout/TypeFeatures.h>
+#include <osmscout/feature/AccessFeature.h>
+#include <osmscout/feature/AccessRestrictedFeature.h>
+#include <osmscout/feature/AddressFeature.h>
+#include <osmscout/feature/AdminLevelFeature.h>
+#include <osmscout/feature/BridgeFeature.h>
+#include <osmscout/feature/BuildingFeature.h>
+#include <osmscout/feature/ClockwiseDirectionFeature.h>
+#include <osmscout/feature/ColorFeature.h>
+#include <osmscout/feature/ConstructionYearFeature.h>
+#include <osmscout/feature/DestinationFeature.h>
+#include <osmscout/feature/EleFeature.h>
+#include <osmscout/feature/EmbankmentFeature.h>
+#include <osmscout/feature/FromToFeature.h>
+#include <osmscout/feature/GradeFeature.h>
+#include <osmscout/feature/IsInFeature.h>
+#include <osmscout/feature/LanesFeature.h>
+#include <osmscout/feature/LayerFeature.h>
+#include <osmscout/feature/LocationFeature.h>
+#include <osmscout/feature/MaxSpeedFeature.h>
+#include <osmscout/feature/NameFeature.h>
+#include <osmscout/feature/NameAltFeature.h>
+#include <osmscout/feature/NameShortFeature.h>
+#include <osmscout/feature/NetworkFeature.h>
+#include <osmscout/feature/OpeningHoursFeature.h>
+#include <osmscout/feature/OperatorFeature.h>
+#include <osmscout/feature/PhoneFeature.h>
+#include <osmscout/feature/PostalCodeFeature.h>
+#include <osmscout/feature/RefFeature.h>
+#include <osmscout/feature/RoundaboutFeature.h>
+#include <osmscout/feature/SidewayFeature.h>
+#include <osmscout/feature/TunnelFeature.h>
+#include <osmscout/feature/WebsiteFeature.h>
+#include <osmscout/feature/WidthFeature.h>
 
 #include <osmscout/system/Assert.h>
 
 #include <osmscout/ost/Parser.h>
 #include <osmscout/ost/Scanner.h>
 
-#include <osmscout/util/File.h>
-#include <osmscout/util/Logger.h>
+#include <osmscout/log/Logger.h>
 #include <osmscout/util/Number.h>
 
-namespace osmscout {
+#include <osmscout/io/File.h>
+
+  namespace osmscout {
 
   const char* TypeConfig::FILE_TYPES_DAT="types.dat";
 
@@ -703,6 +736,9 @@ namespace osmscout {
     RegisterFeature(std::make_shared<FromToFeature>());
     RegisterFeature(std::make_shared<ColorFeature>());
 
+    featureOpeningHours=std::make_shared<OpeningHoursFeature>();
+    RegisterFeature(featureOpeningHours);
+
     // Make sure, that this is always registered first.
     // It assures that id 0 is always reserved for typeIgnore
     typeInfoIgnore=std::make_shared<TypeInfo>("");
@@ -882,7 +918,7 @@ namespace osmscout {
     }
 
     // Something that has a name and is a POI automatically gets the
-    // postal code, location, address, website and phone features, too.
+    // postal code, location, address, website, phone and opening hours features, too.
     if (typeInfo->HasFeature(NameFeature::NAME) &&
         typeInfo->GetIndexAsPOI()) {
       if (!typeInfo->HasFeature(PostalCodeFeature::NAME)) {
@@ -900,6 +936,9 @@ namespace osmscout {
       if (!typeInfo->HasFeature(PhoneFeature::NAME)) {
         typeInfo->AddFeature(featurePhone);
       }
+      if (!typeInfo->HasFeature(OpeningHoursFeature::NAME)) {
+        typeInfo->AddFeature(featureOpeningHours);
+      }
     }
 
     typeInfo->SetIndex(types.size());
