@@ -31,7 +31,7 @@
 #include <osmscout/system/Assert.h>
 
 #include <osmscout/util/Geometry.h>
-#include <osmscout/util/Logger.h>
+#include <osmscout/log/Logger.h>
 #include <osmscout/util/StopClock.h>
 
 //#define DEBUG_ROUTING
@@ -101,10 +101,14 @@ namespace osmscout {
     }
 
     for (auto& handle : handles) {
-      handle.routingDatabase->Close();
-      handle.routingDatabase.reset();
-      handle.router->Close();
-      handle.router.reset();
+      if (handle.routingDatabase) {
+        handle.routingDatabase->Close();
+        handle.routingDatabase.reset();
+      }
+      if (handle.router) {
+        handle.router->Close();
+        handle.router.reset();
+      }
       handle.profile.reset();
     }
 
@@ -123,7 +127,7 @@ namespace osmscout {
       if (res.IsValid()) {
         return RoutePositionResult(RoutePosition(res.GetRoutePosition().GetObjectFileRef(),
                                                  res.GetRoutePosition().GetNodeIndex(),
-                                                 /*database*/ dbId),
+                                                 /*db*/ dbId),
                                    res.GetDistance());
       }
     }
@@ -142,7 +146,7 @@ namespace osmscout {
       if (position.IsValid() && position.GetDistance() < closestPosition.GetDistance()) {
         closestPosition=RoutePositionResult(RoutePosition(position.GetRoutePosition().GetObjectFileRef(),
                                                           position.GetRoutePosition().GetNodeIndex(),
-                                                          /*database*/ handle.dbId),
+                                                          /*db*/ handle.dbId),
                                             position.GetDistance());
       }
     }
@@ -194,6 +198,12 @@ namespace osmscout {
     return handles[database].profile->GetCosts(*way,wayLength);
   }
 
+  double MultiDBRoutingService::GetUTurnCost(const MultiDBRoutingState& /*state*/, const DatabaseId database)
+  {
+    assert(handles.size()>database);
+    return handles[database].profile->GetUTurnCost();
+  }
+
   double MultiDBRoutingService::GetEstimateCosts(const MultiDBRoutingState& /*state*/,
                                                  const DatabaseId database,
                                                  const Distance &targetDistance)
@@ -376,6 +386,7 @@ namespace osmscout {
 
   RoutingResult MultiDBRoutingService::CalculateRoute(const RoutePosition &start,
                                                       const RoutePosition &target,
+                                                      const std::optional<osmscout::Bearing> &bearing,
                                                       const RoutingParameter &parameter)
   {
     RoutingResult result;
@@ -391,21 +402,20 @@ namespace osmscout {
       return service->CalculateRoute(*handles[dbId].profile,
                                      start,
                                      target,
+                                     bearing,
                                      parameter);
     }
 
     // start and target databases are different, try to find common route nodes
     if (start.GetDatabaseId()>=handles.size() ||
         !handles[start.GetDatabaseId()].database) {
-      log.Error() << "Can't find start database " << start.GetDatabaseId();
+      log.Error() << "Can't find start db " << start.GetDatabaseId();
       return result;
     }
 
-    // DatabaseRef database1=handles[start.GetDatabaseId()].database;
-
     if (target.GetDatabaseId()>=handles.size() ||
         !handles[target.GetDatabaseId()].database) {
-      log.Error() << "Can't find target database " << target.GetDatabaseId();
+      log.Error() << "Can't find target db " << target.GetDatabaseId();
       return result;
     }
 
@@ -413,6 +423,7 @@ namespace osmscout {
     return AbstractRoutingService<MultiDBRoutingState>::CalculateRoute(state,
                                                                        start,
                                                                        target,
+                                                                       bearing,
                                                                        parameter);
   }
 
@@ -455,10 +466,12 @@ namespace osmscout {
 
         partialResult=CalculateRoute(fromRoutePosition,
                                      toRoutePosition,
+                                     std::nullopt,
                                      parameter);
         if (!partialResult.Success()) {
           result.GetRoute().Clear();
-
+          result.ClearSectionLengths();
+            
           return result;
         }
 
@@ -469,6 +482,7 @@ namespace osmscout {
         }
 
         result.GetRoute().Append(partialResult.GetRoute());
+        result.AppendSectionLength(partialResult.GetRoute().Entries().size());
       }
 
       return result;
